/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <glib.h>
#include <gio/gio.h>
#include <atompub/atom.h>
#include <atompub/atom-glib.h>

static GFile *
gio_iri_to_file (AtomCtx *ctx, IRI *iri)
{
  gchar *root = atom_config_get_str (ctx, "gio", "root");
  gchar *path = iri_get_path (iri);
  gchar *filename = g_build_filename (root, path, NULL);
  GFile *file = g_file_new_for_path (filename);
  g_free (root);
  g_free (filename);
  return file;
}

static AtomEntry *
gio_file_to_atom (AtomCtx *ctx, GFile *file)
{
  GError *error = NULL;
  gchar *data;
  gsize len;
  AtomEntry *atom;
  error = NULL;
  if (!g_file_load_contents (file, NULL, &data, &len, NULL, &error))
    {
      AtomError *aerr = atom_error_new_from_gerror (error);
      atom_error_set (ctx, aerr);
      g_error_free (error);
      return NULL;
    }
  atom = atom_entry_new_data_len (data, len);
  g_free (data);
  return atom;
}

static AtomEntry *
gio_atom_retrieve_entry (AtomCtx *ctx, IRI *iri)
{
  GFile *file;
  AtomEntry *atom;
  file = gio_iri_to_file (ctx, iri);
  atom = gio_file_to_atom (ctx, file);
  g_object_unref (file);
  return atom;
}

static void
gio_enumerate_entries (AtomCtx *ctx, AtomEntry ***entries, size_t *len)
{
  GFile *dir;
  GFileEnumerator *enumerator;
  GFileInfo *info;
  GFile *file;
  gchar *root;
  gchar *name;
  gchar *filename;
  GPtrArray *array;
  root = atom_config_get_str (ctx, "gio", "root");
  dir = g_file_new_for_path (root);
  enumerator = g_file_enumerate_children (dir, G_FILE_ATTRIBUTE_STANDARD_NAME,
					  G_FILE_QUERY_INFO_NONE, NULL, NULL);
  array = g_ptr_array_new ();
  while ((info = g_file_enumerator_next_file (enumerator, NULL, NULL)) != NULL)
    {
      name = g_file_info_get_name (info);
      filename = g_build_filename (root, name, NULL);
      g_object_unref (info);
      file = g_file_new_for_path (filename);
      g_ptr_array_add (array, gio_file_to_atom (ctx, file));
      g_object_unref (file);
      g_free (filename);
    }
  g_object_unref (enumerator);
  g_object_unref (dir);
  g_free (root);
  if (entries)
    *entries = array->pdata;
  if (len)
    *len = array->len;
  g_ptr_array_free (array, FALSE);
}

AtomBackend *
gio_backend (void)
{
  AtomBackend *backend;
  backend = atom_backend_new ();
  atom_backend_retrieve_entry_set (backend, gio_atom_retrieve_entry);
  atom_backend_enumerate_entries_set (backend, gio_enumerate_entries);
  return backend;
}
